@@ -203,7 +203,7 @@ as_matrix.igraph <- function(.data,
   if ((!is.null(twomode) && twomode) | (is.null(twomode) & is_twomode(.data))) {
     if (is_weighted(.data) | is_signed(.data)) {
       mat <- igraph::as_biadjacency_matrix(.data, sparse = FALSE,
-                                           attr = igraph::edge_attr_names(.data)[[1]])
+                                           attr = ifelse(is_weighted(.data), "weight", NULL))
     } else {
       mat <- igraph::as_biadjacency_matrix(.data, sparse = FALSE,
                                            attr = NULL)
@@ -211,7 +211,7 @@ as_matrix.igraph <- function(.data,
   } else {
     if (is_weighted(.data) | is_signed(.data)) {
       mat <- igraph::as_adjacency_matrix(.data, sparse = FALSE,
-                                         attr = igraph::edge_attr_names(.data)[[1]])
+                                         attr = ifelse(is_weighted(.data), "weight", NULL))
     } else {
       mat <- igraph::as_adjacency_matrix(.data, sparse = FALSE,
                                          attr = NULL)
